package de.intevation.flys.client.client.ui;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.core.client.GWT;

import com.smartgwt.client.widgets.Canvas;
import com.smartgwt.client.widgets.events.ClickEvent;
import com.smartgwt.client.widgets.events.ClickHandler;

import de.intevation.flys.client.client.event.HasStepForwardHandlers;
import de.intevation.flys.client.client.event.StepForwardEvent;
import de.intevation.flys.client.client.event.StepForwardHandler;
import de.intevation.flys.client.shared.model.Data;
import de.intevation.flys.client.shared.model.DataList;


/**
 * An abstract UIProvider that provides some basic methods.
 *
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public abstract class AbstractUIProvider
implements   UIProvider, HasStepForwardHandlers, ClickHandler
{
    /** The StepForwardHandlers.*/
    protected List<StepForwardHandler> forwardHandlers;


    /**
     * Creates a new UIProvider instance of this class.
     */
    public AbstractUIProvider() {
        forwardHandlers = new ArrayList<StepForwardHandler>();
    }


    /**
     * Appends a StepForwardHandler that wants to listen to StepForwardEvents.
     *
     * @param handler A new StepForwardHandler.
     */
    public void addStepForwardHandler(StepForwardHandler handler) {
        if (handler != null) {
            forwardHandlers.add(handler);
        }
    }


    /**
     * This method is called after the user has clicked on the 'next' button to
     * step to the next state.
     *
     * @param e The StepForwardEvent.
     */
    protected void fireStepForwardEvent(StepForwardEvent e) {
        GWT.log("AbstractUIProvider - fireStepForwardEvent() handlers: " + forwardHandlers.size());
        for (StepForwardHandler handler: forwardHandlers) {
            handler.onStepForward(e);
        }
    }


    /**
     * This method is used to listen to click events on the 'next' button. The
     * fireStepForwardEvent() method is called here.
     *
     * @param e The click event.
     */
    public void onClick(ClickEvent e) {
        Data[] data = getData();

        fireStepForwardEvent(new StepForwardEvent(data));
    }


    /**
     * This method needs to be implemented by concrete subclasses. It should
     * create a new Canvas object with a representation of <i>data</i>.
     *
     * @param data The data that should be displayed.
     *
     * @return a Canvas object that displays <i>data</i>.
     */
    public abstract Canvas create(DataList data);


    /**
     * This method needs to be implemented by concrete subclasses. It should
     * return the selected data.
     *
     * @return the selected data.
     */
    protected abstract Data[] getData();
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
